// This class turns a non-blocking queue into a blocking queue by
// catching the exception ImpossibleNow and re-trying. In other
// words, this code uses busy-waiting, and is very inefficient.

public class BusyWaitingWrapper<E> implements SimpleBlockingQueueInterface<E> {

  // The queue object to which we are mediating access.
  private final SimpleNonBlockingQueueInterface<E> q;

  BusyWaitingWrapper (SimpleNonBlockingQueueInterface<E> q)
  {
    this.q = q;
  }

  public void put (E element)
  {
    while (true)
      try {
        q.put(element);
        return;
      } catch (ImpossibleNow e) { /* retry */ }
  }

  public E take ()
  {
    while (true)
      try {
        return q.take();
      } catch (ImpossibleNow e) { /* retry */ }
  }

}
