public class TestQueues {

  static final int T = 4;
  static final int N = 10000;

  public static void test (final SimpleBlockingQueueInterface<Integer> q) throws InterruptedException
  {
    long startTime = System.nanoTime();
    // Create T producers.
    for (int p = 0; p < T; p++) {
      final int id = p;
      new Thread (new Runnable () {
	public void run () {
	  // Each producer puts in N elements.
	  for (int i = 0; i < N; i++)
	    q.put(id);
          System.out.printf("Producer %d done.\n", id);
	}
      }).start();
    }
    // Create T consumers.
    Thread[] consumer = new Thread [T];
    for (int c = 0; c < T; c++) {
      final int id = c;
      consumer[c] = new Thread (new Runnable () {
	public void run () {
	  // Each consumer takes N elements.
	  for (int i = 0; i < N; i++)
	    q.take();
          System.out.printf("Consumer %d done.\n", id);
	}
      });
      consumer[c].start();
    }
    // Wait for the consumers to terminate.
    for (int c = 0; c < T; c++)
      consumer[c].join();
    // Done.
    long endTime = System.nanoTime();
    System.out.println("All consumers are done.");
    System.out.printf("Elapsed time: %.2fs\n", ((double) (endTime - startTime)) / 1000000000);
  }

  public static void main (String[] args) throws InterruptedException
  {
    // Test a sequential queue (no locks at all). This test may fail!
    // E.g. a producer or consumer could remain blocked forever.
    test(new BusyWaitingWrapper<Integer> (new SimplisticQueue<Integer> (27)));

    // Test a simplistic concurrent queue (based on locks and busy waiting).
    // test(new BusyWaitingWrapper<Integer> (new SimplisticConcurrentQueue<Integer> (27)));

    // Test a simple concurrent queue (based on locks and condition variables).
    // test(new SimpleConcurrentQueue<Integer> (27));

  }

  // A purely sequential test of the simplistic queue.
  public static void sequentialTest () throws ImpossibleNow
  {
    SimplisticQueue<Integer> q = new SimplisticQueue<Integer> (27);
    int i = 0;
    // Put 13 elements.
    for (int k = 0; k < 13; k++)
      q.put(i++);
    // Repeatedly...
    for (int j = 0; j < 1000; j++) {
      // Put 13 more.
      for (int k = 0; k < 13; k++)
	q.put(i++);
      // Take 13 more.
      for (int k = 0; k < 13; k++)
	System.out.println(q.take());
    }
    // Take the last 13.
    for (int k = 0; k < 13; k++)
      System.out.println(q.take());
    // Check for emptiness.
    try {
      q.take();
      System.out.println("Bizarre, the queue is not empty.");
    } catch (ImpossibleNow e) {
      System.out.println("OK");
    }
  }

}
